<?php
abstract class AbstractCart {
	
	private $items;
	
	private static $cart = null;
	
	/**
	 * Factory metoda, ktera vyrobi nakupni kosik pozadovaneho typu
	 * @param $cartType - typ kosiku
	 * @return referenci na instanci kosiku
	 */
	static protected function &getACart($cartType) {
		// kosik muze byt ulozen v $_SESSION
		if (isset($_SESSION['cart']) && is_a($_SESSION['cart'], $cartType))
			self::$cart = &$_SESSION['cart'];
		else if (self::$cart == null && $cartType != __CLASS__) {
			self::$cart = new $cartType();
			$_SESSION['cart'] = &self::$cart;
		}
		else
			throw new CartException('Nelze vytvorit instanci kosiku. Pozadovana trida: '.$cartType.'.');
		return self::$cart;
	}
	
	protected function __construct() {
		$this->items = array();
	}
	
	public function addToCart($item, $qty) {
		// if item is aleready in cart, add requested qty
		if (isset($this->items[$item->getId()]))
			$this->items[$item->getId()]['qty'] += $qty;
		// else save reference and qty
		else
			$this->items[$item->getId()] = array('item' => $item, 'qty' => $qty);
	}
	
	public function removeFromCart($item, $qty) {
		// substract from item quantity
		if (isset($this->items[$item->getId()]))
			$this->items[$item->getId()]['qty'] -= $qty;
		// if quantity is less or equal zero, unset item at all
		if ($this->items[$item->getId()]['qty'] <= 0)
			unset($this->items[$item->getId()]);
	}
	
	public function getItemQty($item) {
		if (!empty($this->items[$item->getId()]['qty']))
			return $this->items[$item->getId()]['qty'];
		else
			return 0;
	}
	
	public function __toString() {
		$result = '<table><tr><th>Item</th><th>Qty</th></tr>';
		foreach($this->items as &$item)
			$result .= '<tr><td>'.$item['item'].'</td><td>'.$item['qty'].'</td></tr>'; 
		$result .= '</table>';
		return $result;
	}
}

class CartException extends Exception {
	
	public function __construct($msg) {
		parent::__construct($msg);
	}
	
	public function __toString() {
		return $this->message;
	}
}
?>